# 基础定义

## 语言特点

- Java为纯面向对象的语言。它能够直接反应现实生活中的对象
- 具有跨平台性。java利用Java虚拟机运行字节码，编译后的程序可在其它平台运行
- Java为解释型语言，编译器把Java代码编译成平台无关的中间代码，然后在JVM上解释运行，具有很好的可移植性
- Java提供了很多内置类库。如对多线程支持，对网络通信支持，最重要的一点是提供了垃圾回收器
- Java具有较好的安全性和健壮性。Java提供了异常处理和垃圾回收机制，去除了C++中难以理解的指针特性
- Java语言提供了对Web应用开发的支持

## 面向对象三大特性

- **继承**

  对象的一个新类可以从现有的类中派生，派生类可以从它的基类那继承方法和实例变量，且派生类可以修改或新增新的方法使之更适合特殊的需求

- **封装**

  将客观事物抽象成类，每个类可以把自身数据和方法只让可信的类或对象操作，对不可信的进行信息隐藏

- **多态**

  允许不同类的对象对同一消息作出响应。不同对象调用相同方法即使参数也相同，最终表现行为是不一样的

## JDK与JRE的区别

- **JDK**

  Java开发工具包（Java Development Kit），提供了Java的开发环境和运行环境

- **JRE**

  Java运行环境(Java Runtime Environment)，提供了Java运行所需的环境

## 访问修饰符

- default: 默认访问修饰符，在同一包内可见
- private: 在同一类内可见，不能修饰类
- protected : 对同一包内的类和所有子类可见，不能修饰类
- public: 对所有类可见

## 抽象类和接口的区别

- 实现接口的关键字为implements,继承抽象类的关键字为extends。一个类可以实现多个接口，只能继承一个抽象类。
- 抽象类可以有构造方法，接口中不能有构造方法。
- 抽象类中可以有普通成员变量，接口中没有普通成员变量，它的变量只能是公共的静态的常量
- 接口只能做方法声明(Java8中接口也可有默认方法)，抽象类中可以作方法声明，也可以做方法实现。
- 当子类和父类之间存在逻辑上的层次结构，推荐使用抽象类，有利于功能的累积。当功能不需要，希望支持差别较大的两个或更多对象间的特定交互行为，推荐使用接口。使用接口能降低软件系统的耦合度，便于日后维护或添加删除方法

## 多态的机制

- 方法的重载

  重载是指同一个类中有多个同名的方法，但这些方法有不同的参数，在编译期间就可以确定调用哪个方法

- 方法覆盖

  父类应用指向子类对象

## final、finally和finalize的区别

- final用于声明属性、方法和类，分别表示属性不可变、方法不可覆盖、类不可继承
- finally作为异常处理的一部分，只能在try/catch语句中使用，finally附带一个语句块用来表示这个语句最终一定被执行，经常被用在需要释放资源的情况下
- finalize是Object类的一个方法，在垃圾收集器执行的时候会调用被回收对象的finalize()方法。当垃圾回收器准备好释放对象占用空间时，首先会调用finalize()方法，并在下一次垃圾回收动作发生时真正回收对象占用的内存

## static作用

- 为某种特定数据类型或对象分配与创建对象个数无关的单一的存储空间
- 使得某个方法或属性与类而不是对象关联在一起，即在不创建对象的情况下可通过类直接调用方法或使用类的属性

## String、StringBuffer、StringBuilder的区别

- **String**
  - String类利用final修饰的字符数组进行字符串保存，是一个不可变的类，一旦创建就不可以修改。
  - String是final类，不能被继承
  - String实现了equals()方法和hashCode()方法
- **StringBuffer**
  - 继承自AbstractStringBuilder，是可变类。
  - StringBuffer是线程安全的
  - 可以通过append方法动态构造数据。
- **StringBuilder**
  - 继承自AbstractStringBuilder，是可变类。
  - StringBuilder是非线性安全的。
  - 执行效率比StringBuffer高。

## 运算符==与equals方法的区别

​	**==** 比较的是引用，equals比较的是内容

- 如果变量是基础数据类型，**==** 用于比较其对应值是否相等。如果变量指向的是对象,**==** 用于比较两个对象是否指向同一块存储空间
- equals是Object类提供的方法之一，每个Java类都继承自Object类，所以每个对象都具有equals这个方法。Object类中定义的equals方法内部是直接调用 **==** 比较对象的。但通过覆盖的方法可以让它不是比较引用而是比较数据内容

## 反射机制

​	Java反射机制是指在程序的运行过程中可以构造任意一个类的对象、获取任意一个类的成员变量和成员方法、获取任意一个对象所属的类信息、调用任意一个对象的属性和方法。反射机制使得Java具有动态获取程序信息和动态调用对象方法的能力。可以通过以下类调用反射API

- Class类：可获得类属性方法
- Field类：获得类的成员变量
- Method类：获取类的方法信息
- Construct类：获取类的构造方法等信息

## 异常的分类

- Java异常分为Error（程序无法处理的错误），和Exception（程序本身可以处理的异常）。这两个类均继承Throwable
- Error常见的有StackOverFlowError,OutOfMemoryError等
- Exception可分为运行时异常和非运行时异常。对于运行时异常，可以利用try catch的方式进行处理，也可以不处理。对于非运行时异常，必须处理，不处理的话程序无法通过编译

## throw与throws

- throw一般是用在方法体的内部，由开发者定义当程序语句出现问题后主动抛出一个异常
- throws一般用于方法声明上，代表该方法可能会抛出的异常列表

## 泛型

​	泛型，即“参数化类型”，解决不确定对象具体类型的问题。在编译阶段有效。在泛型使用过程中，操作的数据类型被指定为一个参数，这种参数类型在类中称为泛型类、接口中称为泛型接口和方法中称为泛型方法

## 基本数据类型

- byte: 占用1个字节，取值范围-128 ~ 127
- short: 占用2个字节，取值范围-2^15^ ~ 2^15^-1
- int：占用4个字节，取值范围-2^31^ ~ 2^31^-1
- long：占用8个字节
- float：占用4个字节
- double：占用8个字节
- char: 占用2个字节
- boolean：占用大小根据实现虚拟机不同有所差异

## 自动装箱拆箱

​	对于Java基本数据类型，均对应一个包装类。装箱就是自动将基本数据类型转换为包装器类型，如int->Integer，拆箱就是自动将包装器类型转换为基本数据类型，如Integer->int

## 内部类及其作用

- 成员内部类：作为成员对象的内部类。可以访问private及以上外部类的属性和方法。外部类想要访问内部类属性或方法时，必须要创建一个内部类对象，然后通过该对象访问内部类的属性或方法。外部类也可访问private修饰的内部类属性。
- 局部内部类：存在于方法中的内部类。访问权限类似局部变量，只能访问外部类的final变量。
- 匿名内部类：只能使用一次，没有类名，只能访问外部类的final变量。
- 静态内部类：类似类的静态成员变量。

## 序列化与反序列化

- 序列化：将java对象转化为字节序列，由此可以通过网络对象进行传输。
- 反序列化：将字节序列转化为java对象。
- 具体实现：实现Serializable接口，或实现Externalizable接口中的writeExternal()与readExternal()方法。

# 集合

## List

**List是一个有序队列，在JAVA中有两种实现方式**

- ArrayList 使用数组实现，是容量可变的非线程安全列表，随机访问快，集合扩容时会创建更大的数组，把原有数组复制到新数组
- LinkedList 本质是双向链表，与 ArrayList 相比插入和删除速度更快，但随机访问元素慢

## Set

**Set 即集合，该数据结构不允许元素重复且无序。JAVA对Set有三种实现方式**

- HashSet 通过 HashMap 实现，HashMap 的 Key 即 HashSet 存储的元素，Value系统自定义一个名为 PRESENT 的 Object 类型常量。判断元素是否相同时，先比较hashCode，相同后再利用equals比较，查询O(1)
- LinkedHashSet 继承自 HashSet，通过 LinkedHashMap 实现，使用双向链表维护元素插入顺序
- TreeSet 通过 TreeMap 实现的，底层数据结构是红黑树，添加元素到集合时按照比较规则将其插入合适的位置，保证插入后的集合仍然有序。查询O(logn)

## HashMap

- JDK8 之前底层实现是数组 + 链表，JDK8 改为数组 + 链表/红黑树。主要成员变量包括存储数据的 table 数组、元素数量 size、加载因子 loadFactor。 HashMap 中数据以键值对的形式存在，键对应的 hash 值用来计算数组下标，如果两个元素 key 的 hash 值一样，就会发生哈希冲突，被放到同一个链表上。
- table 数组记录 HashMap 的数据，每个下标对应一条链表，所有哈希冲突的数据都会被存放到同一条链表，Node/Entry 节点包含四个成员变量：key、value、next 指针和 hash 值。在JDK8后链表超过8会转化为红黑树。
- 若当前数据/总数据容量>负载因子，Hashmap将执行扩容操作。 默认初始化容量为 16，扩容容量必须是 2 的幂次方、最大容量为 1<< 30 、默认加载因子为 0.75。
- 在JDK1.7中，HashMap采用头插法插入元素，因此并发情况下会导致环形链表，产生死循环。虽然JDK1.8采用了尾插法解决了这个问题，但是并发下的put操作也会使前一个key被后一个key覆盖。由于HashMap有扩容机制存在，也存在A线程进行扩容后，B线程执行get方法出现失误的情况。

## 线程安全的基本数据结构

- HashTable: 哈希表的线程安全版，效率低 
- ConcurrentHashMap：哈希表的线程安全版，效率高，用于替代HashTable
- Vector：线程安全版Arraylist
- Stack：线程安全版栈
- BlockingQueue及其子类：线程安全版队列
